# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import functools
import re
from typing import Dict, Optional, Sequence, Tuple, Type, Union
import pkg_resources

from google.api_core.client_options import ClientOptions
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore

from google.cloud.accessapproval_v1.services.access_approval import pagers
from google.cloud.accessapproval_v1.types import accessapproval
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from .transports.base import AccessApprovalTransport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import AccessApprovalGrpcAsyncIOTransport
from .client import AccessApprovalClient


class AccessApprovalAsyncClient:
    """This API allows a customer to manage accesses to cloud resources by
    Google personnel. It defines the following resource model:

    -  The API has a collection of
       [ApprovalRequest][google.cloud.accessapproval.v1.ApprovalRequest]
       resources, named ``approvalRequests/{approval_request}``
    -  The API has top-level settings per Project/Folder/Organization,
       named ``accessApprovalSettings``

    The service also periodically emails a list of recipients, defined
    at the Project/Folder/Organization level in the
    accessApprovalSettings, when there is a pending ApprovalRequest for
    them to act on. The ApprovalRequests can also optionally be
    published to a Pub/Sub topic owned by the customer (contact support
    if you would like to enable Pub/Sub notifications).

    ApprovalRequests can be approved or dismissed. Google personnel can
    only access the indicated resource or resources if the request is
    approved (subject to some exclusions:
    https://cloud.google.com/access-approval/docs/overview#exclusions).

    Note: Using Access Approval functionality will mean that Google may
    not be able to meet the SLAs for your chosen products, as any
    support response times may be dramatically increased. As such the
    SLAs do not apply to any service disruption to the extent impacted
    by Customer's use of Access Approval. Do not enable Access Approval
    for projects where you may require high service availability and
    rapid response by Google Cloud Support.

    After a request is approved or dismissed, no further action may be
    taken on it. Requests with the requested_expiration in the past or
    with no activity for 14 days are considered dismissed. When an
    approval expires, the request is considered dismissed.

    If a request is not approved or dismissed, we call it pending.
    """

    _client: AccessApprovalClient

    DEFAULT_ENDPOINT = AccessApprovalClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = AccessApprovalClient.DEFAULT_MTLS_ENDPOINT

    access_approval_settings_path = staticmethod(
        AccessApprovalClient.access_approval_settings_path
    )
    parse_access_approval_settings_path = staticmethod(
        AccessApprovalClient.parse_access_approval_settings_path
    )
    approval_request_path = staticmethod(AccessApprovalClient.approval_request_path)
    parse_approval_request_path = staticmethod(
        AccessApprovalClient.parse_approval_request_path
    )
    common_billing_account_path = staticmethod(
        AccessApprovalClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        AccessApprovalClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(AccessApprovalClient.common_folder_path)
    parse_common_folder_path = staticmethod(
        AccessApprovalClient.parse_common_folder_path
    )
    common_organization_path = staticmethod(
        AccessApprovalClient.common_organization_path
    )
    parse_common_organization_path = staticmethod(
        AccessApprovalClient.parse_common_organization_path
    )
    common_project_path = staticmethod(AccessApprovalClient.common_project_path)
    parse_common_project_path = staticmethod(
        AccessApprovalClient.parse_common_project_path
    )
    common_location_path = staticmethod(AccessApprovalClient.common_location_path)
    parse_common_location_path = staticmethod(
        AccessApprovalClient.parse_common_location_path
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            AccessApprovalAsyncClient: The constructed client.
        """
        return AccessApprovalClient.from_service_account_info.__func__(AccessApprovalAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            AccessApprovalAsyncClient: The constructed client.
        """
        return AccessApprovalClient.from_service_account_file.__func__(AccessApprovalAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variabel is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return AccessApprovalClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> AccessApprovalTransport:
        """Returns the transport used by the client instance.

        Returns:
            AccessApprovalTransport: The transport used by the client instance.
        """
        return self._client.transport

    get_transport_class = functools.partial(
        type(AccessApprovalClient).get_transport_class, type(AccessApprovalClient)
    )

    def __init__(
        self,
        *,
        credentials: ga_credentials.Credentials = None,
        transport: Union[str, AccessApprovalTransport] = "grpc_asyncio",
        client_options: ClientOptions = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the access approval client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, ~.AccessApprovalTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (ClientOptions): Custom options for the client. It
                won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = AccessApprovalClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

    async def list_approval_requests(
        self,
        request: Union[accessapproval.ListApprovalRequestsMessage, dict] = None,
        *,
        parent: str = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListApprovalRequestsAsyncPager:
        r"""Lists approval requests associated with a project,
        folder, or organization. Approval requests can be
        filtered by state (pending, active, dismissed). The
        order is reverse chronological.


        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_list_approval_requests():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.ListApprovalRequestsMessage(
                )

                # Make the request
                page_result = client.list_approval_requests(request=request)

                # Handle the response
                for response in page_result:
                    print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.ListApprovalRequestsMessage, dict]):
                The request object. Request to list approval requests.
            parent (:class:`str`):
                The parent resource. This may be
                "projects/{project}",
                "folders/{folder}", or
                "organizations/{organization}".

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.services.access_approval.pagers.ListApprovalRequestsAsyncPager:
                Response to listing of
                ApprovalRequest objects.
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([parent])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = accessapproval.ListApprovalRequestsMessage(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if parent is not None:
            request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.list_approval_requests,
            default_retry=retries.Retry(
                initial=0.1,
                maximum=60.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=600.0,
            ),
            default_timeout=600.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__aiter__` convenience method.
        response = pagers.ListApprovalRequestsAsyncPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_approval_request(
        self,
        request: Union[accessapproval.GetApprovalRequestMessage, dict] = None,
        *,
        name: str = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> accessapproval.ApprovalRequest:
        r"""Gets an approval request. Returns NOT_FOUND if the request does
        not exist.


        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_get_approval_request():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.GetApprovalRequestMessage(
                )

                # Make the request
                response = client.get_approval_request(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.GetApprovalRequestMessage, dict]):
                The request object. Request to get an approval request.
            name (:class:`str`):
                The name of the approval request to retrieve. Format:
                "{projects|folders|organizations}/{id}/approvalRequests/{approval_request}"

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.types.ApprovalRequest:
                A request for the customer to approve
                access to a resource.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = accessapproval.GetApprovalRequestMessage(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_approval_request,
            default_retry=retries.Retry(
                initial=0.1,
                maximum=60.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=600.0,
            ),
            default_timeout=600.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def approve_approval_request(
        self,
        request: Union[accessapproval.ApproveApprovalRequestMessage, dict] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> accessapproval.ApprovalRequest:
        r"""Approves a request and returns the updated ApprovalRequest.

        Returns NOT_FOUND if the request does not exist. Returns
        FAILED_PRECONDITION if the request exists but is not in a
        pending state.


        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_approve_approval_request():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.ApproveApprovalRequestMessage(
                )

                # Make the request
                response = client.approve_approval_request(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.ApproveApprovalRequestMessage, dict]):
                The request object. Request to approve an
                ApprovalRequest.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.types.ApprovalRequest:
                A request for the customer to approve
                access to a resource.

        """
        # Create or coerce a protobuf request object.
        request = accessapproval.ApproveApprovalRequestMessage(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.approve_approval_request,
            default_timeout=600.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def dismiss_approval_request(
        self,
        request: Union[accessapproval.DismissApprovalRequestMessage, dict] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> accessapproval.ApprovalRequest:
        r"""Dismisses a request. Returns the updated ApprovalRequest.

        NOTE: This does not deny access to the resource if another
        request has been made and approved. It is equivalent in effect
        to ignoring the request altogether.

        Returns NOT_FOUND if the request does not exist.

        Returns FAILED_PRECONDITION if the request exists but is not in
        a pending state.


        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_dismiss_approval_request():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.DismissApprovalRequestMessage(
                )

                # Make the request
                response = client.dismiss_approval_request(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.DismissApprovalRequestMessage, dict]):
                The request object. Request to dismiss an approval
                request.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.types.ApprovalRequest:
                A request for the customer to approve
                access to a resource.

        """
        # Create or coerce a protobuf request object.
        request = accessapproval.DismissApprovalRequestMessage(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.dismiss_approval_request,
            default_timeout=600.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def get_access_approval_settings(
        self,
        request: Union[accessapproval.GetAccessApprovalSettingsMessage, dict] = None,
        *,
        name: str = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> accessapproval.AccessApprovalSettings:
        r"""Gets the settings associated with a project, folder,
        or organization.


        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_get_access_approval_settings():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.GetAccessApprovalSettingsMessage(
                )

                # Make the request
                response = client.get_access_approval_settings(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.GetAccessApprovalSettingsMessage, dict]):
                The request object. Request to get access approval
                settings.
            name (:class:`str`):
                The name of the AccessApprovalSettings to retrieve.
                Format:
                "{projects|folders|organizations}/{id}/accessApprovalSettings"

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.types.AccessApprovalSettings:
                Settings on a
                Project/Folder/Organization related to
                Access Approval.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = accessapproval.GetAccessApprovalSettingsMessage(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.get_access_approval_settings,
            default_retry=retries.Retry(
                initial=0.1,
                maximum=60.0,
                multiplier=1.3,
                predicate=retries.if_exception_type(
                    core_exceptions.ServiceUnavailable,
                ),
                deadline=600.0,
            ),
            default_timeout=600.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def update_access_approval_settings(
        self,
        request: Union[accessapproval.UpdateAccessApprovalSettingsMessage, dict] = None,
        *,
        settings: accessapproval.AccessApprovalSettings = None,
        update_mask: field_mask_pb2.FieldMask = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> accessapproval.AccessApprovalSettings:
        r"""Updates the settings associated with a project, folder, or
        organization. Settings to update are determined by the value of
        field_mask.


        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_update_access_approval_settings():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.UpdateAccessApprovalSettingsMessage(
                )

                # Make the request
                response = client.update_access_approval_settings(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.UpdateAccessApprovalSettingsMessage, dict]):
                The request object. Request to update access approval
                settings.
            settings (:class:`google.cloud.accessapproval_v1.types.AccessApprovalSettings`):
                The new AccessApprovalSettings.
                This corresponds to the ``settings`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (:class:`google.protobuf.field_mask_pb2.FieldMask`):
                The update mask applies to the settings. Only the top
                level fields of AccessApprovalSettings
                (notification_emails & enrolled_services) are supported.
                For each field, if it is included, the currently stored
                value will be entirely overwritten with the value of the
                field passed in this request.

                For the ``FieldMask`` definition, see
                https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#fieldmask
                If this field is left unset, only the
                notification_emails field will be updated.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.types.AccessApprovalSettings:
                Settings on a
                Project/Folder/Organization related to
                Access Approval.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([settings, update_mask])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = accessapproval.UpdateAccessApprovalSettingsMessage(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if settings is not None:
            request.settings = settings
        if update_mask is not None:
            request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.update_access_approval_settings,
            default_timeout=600.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("settings.name", request.settings.name),)
            ),
        )

        # Send the request.
        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    async def delete_access_approval_settings(
        self,
        request: Union[accessapproval.DeleteAccessApprovalSettingsMessage, dict] = None,
        *,
        name: str = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Deletes the settings associated with a project,
        folder, or organization. This will have the effect of
        disabling Access Approval for the project, folder, or
        organization, but only if all ancestors also have Access
        Approval disabled. If Access Approval is enabled at a
        higher level of the hierarchy, then Access Approval will
        still be enabled at this level as the settings are
        inherited.


        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_delete_access_approval_settings():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.DeleteAccessApprovalSettingsMessage(
                )

                # Make the request
                client.delete_access_approval_settings(request=request)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.DeleteAccessApprovalSettingsMessage, dict]):
                The request object. Request to delete access approval
                settings.
            name (:class:`str`):
                Name of the AccessApprovalSettings to
                delete.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        request = accessapproval.DeleteAccessApprovalSettingsMessage(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = gapic_v1.method_async.wrap_method(
            self._client._transport.delete_access_approval_settings,
            default_timeout=600.0,
            client_info=DEFAULT_CLIENT_INFO,
        )

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        await rpc(
            request, retry=retry, timeout=timeout, metadata=metadata,
        )

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution(
            "google-cloud-access-approval",
        ).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()


__all__ = ("AccessApprovalAsyncClient",)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .accessapproval import (
    AccessApprovalSettings,
    AccessLocations,
    AccessReason,
    ApprovalRequest,
    ApproveApprovalRequestMessage,
    ApproveDecision,
    DeleteAccessApprovalSettingsMessage,
    DismissApprovalRequestMessage,
    DismissDecision,
    EnrolledService,
    GetAccessApprovalSettingsMessage,
    GetApprovalRequestMessage,
    ListApprovalRequestsMessage,
    ListApprovalRequestsResponse,
    ResourceProperties,
    UpdateAccessApprovalSettingsMessage,
    EnrollmentLevel,
)

__all__ = (
    "AccessApprovalSettings",
    "AccessLocations",
    "AccessReason",
    "ApprovalRequest",
    "ApproveApprovalRequestMessage",
    "ApproveDecision",
    "DeleteAccessApprovalSettingsMessage",
    "DismissApprovalRequestMessage",
    "DismissDecision",
    "EnrolledService",
    "GetAccessApprovalSettingsMessage",
    "GetApprovalRequestMessage",
    "ListApprovalRequestsMessage",
    "ListApprovalRequestsResponse",
    "ResourceProperties",
    "UpdateAccessApprovalSettingsMessage",
    "EnrollmentLevel",
)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from .services.access_approval import AccessApprovalClient
from .services.access_approval import AccessApprovalAsyncClient

from .types.accessapproval import AccessApprovalSettings
from .types.accessapproval import AccessLocations
from .types.accessapproval import AccessReason
from .types.accessapproval import ApprovalRequest
from .types.accessapproval import ApproveApprovalRequestMessage
from .types.accessapproval import ApproveDecision
from .types.accessapproval import DeleteAccessApprovalSettingsMessage
from .types.accessapproval import DismissApprovalRequestMessage
from .types.accessapproval import DismissDecision
from .types.accessapproval import EnrolledService
from .types.accessapproval import GetAccessApprovalSettingsMessage
from .types.accessapproval import GetApprovalRequestMessage
from .types.accessapproval import ListApprovalRequestsMessage
from .types.accessapproval import ListApprovalRequestsResponse
from .types.accessapproval import ResourceProperties
from .types.accessapproval import UpdateAccessApprovalSettingsMessage
from .types.accessapproval import EnrollmentLevel

__all__ = (
    "AccessApprovalAsyncClient",
    "AccessApprovalClient",
    "AccessApprovalSettings",
    "AccessLocations",
    "AccessReason",
    "ApprovalRequest",
    "ApproveApprovalRequestMessage",
    "ApproveDecision",
    "DeleteAccessApprovalSettingsMessage",
    "DismissApprovalRequestMessage",
    "DismissDecision",
    "EnrolledService",
    "EnrollmentLevel",
    "GetAccessApprovalSettingsMessage",
    "GetApprovalRequestMessage",
    "ListApprovalRequestsMessage",
    "ListApprovalRequestsResponse",
    "ResourceProperties",
    "UpdateAccessApprovalSettingsMessage",
)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from google.cloud.accessapproval_v1.services.access_approval.client import (
    AccessApprovalClient,
)
from google.cloud.accessapproval_v1.services.access_approval.async_client import (
    AccessApprovalAsyncClient,
)

from google.cloud.accessapproval_v1.types.accessapproval import AccessApprovalSettings
from google.cloud.accessapproval_v1.types.accessapproval import AccessLocations
from google.cloud.accessapproval_v1.types.accessapproval import AccessReason
from google.cloud.accessapproval_v1.types.accessapproval import ApprovalRequest
from google.cloud.accessapproval_v1.types.accessapproval import (
    ApproveApprovalRequestMessage,
)
from google.cloud.accessapproval_v1.types.accessapproval import ApproveDecision
from google.cloud.accessapproval_v1.types.accessapproval import (
    DeleteAccessApprovalSettingsMessage,
)
from google.cloud.accessapproval_v1.types.accessapproval import (
    DismissApprovalRequestMessage,
)
from google.cloud.accessapproval_v1.types.accessapproval import DismissDecision
from google.cloud.accessapproval_v1.types.accessapproval import EnrolledService
from google.cloud.accessapproval_v1.types.accessapproval import (
    GetAccessApprovalSettingsMessage,
)
from google.cloud.accessapproval_v1.types.accessapproval import (
    GetApprovalRequestMessage,
)
from google.cloud.accessapproval_v1.types.accessapproval import (
    ListApprovalRequestsMessage,
)
from google.cloud.accessapproval_v1.types.accessapproval import (
    ListApprovalRequestsResponse,
)
from google.cloud.accessapproval_v1.types.accessapproval import ResourceProperties
from google.cloud.accessapproval_v1.types.accessapproval import (
    UpdateAccessApprovalSettingsMessage,
)
from google.cloud.accessapproval_v1.types.accessapproval import EnrollmentLevel

__all__ = (
    "AccessApprovalClient",
    "AccessApprovalAsyncClient",
    "AccessApprovalSettings",
    "AccessLocations",
    "AccessReason",
    "ApprovalRequest",
    "ApproveApprovalRequestMessage",
    "ApproveDecision",
    "DeleteAccessApprovalSettingsMessage",
    "DismissApprovalRequestMessage",
    "DismissDecision",
    "EnrolledService",
    "GetAccessApprovalSettingsMessage",
    "GetApprovalRequestMessage",
    "ListApprovalRequestsMessage",
    "ListApprovalRequestsResponse",
    "ResourceProperties",
    "UpdateAccessApprovalSettingsMessage",
    "EnrollmentLevel",
)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import warnings
from typing import Callable, Dict, Optional, Sequence, Tuple, Union

from google.api_core import grpc_helpers
from google.api_core import gapic_v1
import google.auth  # type: ignore
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore

import grpc  # type: ignore

from google.cloud.accessapproval_v1.types import accessapproval
from google.protobuf import empty_pb2  # type: ignore
from .base import AccessApprovalTransport, DEFAULT_CLIENT_INFO


class AccessApprovalGrpcTransport(AccessApprovalTransport):
    """gRPC backend transport for AccessApproval.

    This API allows a customer to manage accesses to cloud resources by
    Google personnel. It defines the following resource model:

    -  The API has a collection of
       [ApprovalRequest][google.cloud.accessapproval.v1.ApprovalRequest]
       resources, named ``approvalRequests/{approval_request}``
    -  The API has top-level settings per Project/Folder/Organization,
       named ``accessApprovalSettings``

    The service also periodically emails a list of recipients, defined
    at the Project/Folder/Organization level in the
    accessApprovalSettings, when there is a pending ApprovalRequest for
    them to act on. The ApprovalRequests can also optionally be
    published to a Pub/Sub topic owned by the customer (contact support
    if you would like to enable Pub/Sub notifications).

    ApprovalRequests can be approved or dismissed. Google personnel can
    only access the indicated resource or resources if the request is
    approved (subject to some exclusions:
    https://cloud.google.com/access-approval/docs/overview#exclusions).

    Note: Using Access Approval functionality will mean that Google may
    not be able to meet the SLAs for your chosen products, as any
    support response times may be dramatically increased. As such the
    SLAs do not apply to any service disruption to the extent impacted
    by Customer's use of Access Approval. Do not enable Access Approval
    for projects where you may require high service availability and
    rapid response by Google Cloud Support.

    After a request is approved or dismissed, no further action may be
    taken on it. Requests with the requested_expiration in the past or
    with no activity for 14 days are considered dismissed. When an
    approval expires, the request is considered dismissed.

    If a request is not approved or dismissed, we call it pending.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends protocol buffers over the wire using gRPC (which is built on
    top of HTTP/2); the ``grpcio`` package must be installed.
    """

    _stubs: Dict[str, Callable]

    def __init__(
        self,
        *,
        host: str = "accessapproval.googleapis.com",
        credentials: ga_credentials.Credentials = None,
        credentials_file: str = None,
        scopes: Sequence[str] = None,
        channel: grpc.Channel = None,
        api_mtls_endpoint: str = None,
        client_cert_source: Callable[[], Tuple[bytes, bytes]] = None,
        ssl_channel_credentials: grpc.ChannelCredentials = None,
        client_cert_source_for_mtls: Callable[[], Tuple[bytes, bytes]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to.
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
                This argument is ignored if ``channel`` is provided.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            channel (Optional[grpc.Channel]): A ``Channel`` instance through
                which to make calls.
            api_mtls_endpoint (Optional[str]): Deprecated. The mutual TLS endpoint.
                If provided, it overrides the ``host`` argument and tries to create
                a mutual TLS channel with client SSL credentials from
                ``client_cert_source`` or application default SSL credentials.
            client_cert_source (Optional[Callable[[], Tuple[bytes, bytes]]]):
                Deprecated. A callback to provide client SSL certificate bytes and
                private key bytes, both in PEM format. It is ignored if
                ``api_mtls_endpoint`` is None.
            ssl_channel_credentials (grpc.ChannelCredentials): SSL credentials
                for the grpc channel. It is ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Optional[Callable[[], Tuple[bytes, bytes]]]):
                A callback to provide client certificate bytes and private key bytes,
                both in PEM format. It is used to configure a mutual TLS channel. It is
                ignored if ``channel`` or ``ssl_channel_credentials`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.

        Raises:
          google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
              creation failed for any reason.
          google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """
        self._grpc_channel = None
        self._ssl_channel_credentials = ssl_channel_credentials
        self._stubs: Dict[str, Callable] = {}

        if api_mtls_endpoint:
            warnings.warn("api_mtls_endpoint is deprecated", DeprecationWarning)
        if client_cert_source:
            warnings.warn("client_cert_source is deprecated", DeprecationWarning)

        if channel:
            # Ignore credentials if a channel was passed.
            credentials = False
            # If a channel was explicitly provided, set it.
            self._grpc_channel = channel
            self._ssl_channel_credentials = None

        else:
            if api_mtls_endpoint:
                host = api_mtls_endpoint

                # Create SSL credentials with client_cert_source or application
                # default SSL credentials.
                if client_cert_source:
                    cert, key = client_cert_source()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )
                else:
                    self._ssl_channel_credentials = SslCredentials().ssl_credentials

            else:
                if client_cert_source_for_mtls and not ssl_channel_credentials:
                    cert, key = client_cert_source_for_mtls()
                    self._ssl_channel_credentials = grpc.ssl_channel_credentials(
                        certificate_chain=cert, private_key=key
                    )

        # The base transport sets the host, credentials and scopes
        super().__init__(
            host=host,
            credentials=credentials,
            credentials_file=credentials_file,
            scopes=scopes,
            quota_project_id=quota_project_id,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
        )

        if not self._grpc_channel:
            self._grpc_channel = type(self).create_channel(
                self._host,
                # use the credentials which are saved
                credentials=self._credentials,
                # Set ``credentials_file`` to ``None`` here as
                # the credentials that we saved earlier should be used.
                credentials_file=None,
                scopes=self._scopes,
                ssl_credentials=self._ssl_channel_credentials,
                quota_project_id=quota_project_id,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )

        # Wrap messages. This must be done after self._grpc_channel exists
        self._prep_wrapped_messages(client_info)

    @classmethod
    def create_channel(
        cls,
        host: str = "accessapproval.googleapis.com",
        credentials: ga_credentials.Credentials = None,
        credentials_file: str = None,
        scopes: Optional[Sequence[str]] = None,
        quota_project_id: Optional[str] = None,
        **kwargs,
    ) -> grpc.Channel:
        """Create and return a gRPC channel object.
        Args:
            host (Optional[str]): The host for the channel to use.
            credentials (Optional[~.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify this application to the service. If
                none are specified, the client will attempt to ascertain
                the credentials from the environment.
            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is mutually exclusive with credentials.
            scopes (Optional[Sequence[str]]): A optional list of scopes needed for this
                service. These are only used when credentials are not specified and
                are passed to :func:`google.auth.default`.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            kwargs (Optional[dict]): Keyword arguments, which are passed to the
                channel creation.
        Returns:
            grpc.Channel: A gRPC channel object.

        Raises:
            google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
              and ``credentials_file`` are passed.
        """

        return grpc_helpers.create_channel(
            host,
            credentials=credentials,
            credentials_file=credentials_file,
            quota_project_id=quota_project_id,
            default_scopes=cls.AUTH_SCOPES,
            scopes=scopes,
            default_host=cls.DEFAULT_HOST,
            **kwargs,
        )

    @property
    def grpc_channel(self) -> grpc.Channel:
        """Return the channel designed to connect to this service.
        """
        return self._grpc_channel

    @property
    def list_approval_requests(
        self,
    ) -> Callable[
        [accessapproval.ListApprovalRequestsMessage],
        accessapproval.ListApprovalRequestsResponse,
    ]:
        r"""Return a callable for the list approval requests method over gRPC.

        Lists approval requests associated with a project,
        folder, or organization. Approval requests can be
        filtered by state (pending, active, dismissed). The
        order is reverse chronological.

        Returns:
            Callable[[~.ListApprovalRequestsMessage],
                    ~.ListApprovalRequestsResponse]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "list_approval_requests" not in self._stubs:
            self._stubs["list_approval_requests"] = self.grpc_channel.unary_unary(
                "/google.cloud.accessapproval.v1.AccessApproval/ListApprovalRequests",
                request_serializer=accessapproval.ListApprovalRequestsMessage.serialize,
                response_deserializer=accessapproval.ListApprovalRequestsResponse.deserialize,
            )
        return self._stubs["list_approval_requests"]

    @property
    def get_approval_request(
        self,
    ) -> Callable[
        [accessapproval.GetApprovalRequestMessage], accessapproval.ApprovalRequest
    ]:
        r"""Return a callable for the get approval request method over gRPC.

        Gets an approval request. Returns NOT_FOUND if the request does
        not exist.

        Returns:
            Callable[[~.GetApprovalRequestMessage],
                    ~.ApprovalRequest]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_approval_request" not in self._stubs:
            self._stubs["get_approval_request"] = self.grpc_channel.unary_unary(
                "/google.cloud.accessapproval.v1.AccessApproval/GetApprovalRequest",
                request_serializer=accessapproval.GetApprovalRequestMessage.serialize,
                response_deserializer=accessapproval.ApprovalRequest.deserialize,
            )
        return self._stubs["get_approval_request"]

    @property
    def approve_approval_request(
        self,
    ) -> Callable[
        [accessapproval.ApproveApprovalRequestMessage], accessapproval.ApprovalRequest
    ]:
        r"""Return a callable for the approve approval request method over gRPC.

        Approves a request and returns the updated ApprovalRequest.

        Returns NOT_FOUND if the request does not exist. Returns
        FAILED_PRECONDITION if the request exists but is not in a
        pending state.

        Returns:
            Callable[[~.ApproveApprovalRequestMessage],
                    ~.ApprovalRequest]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "approve_approval_request" not in self._stubs:
            self._stubs["approve_approval_request"] = self.grpc_channel.unary_unary(
                "/google.cloud.accessapproval.v1.AccessApproval/ApproveApprovalRequest",
                request_serializer=accessapproval.ApproveApprovalRequestMessage.serialize,
                response_deserializer=accessapproval.ApprovalRequest.deserialize,
            )
        return self._stubs["approve_approval_request"]

    @property
    def dismiss_approval_request(
        self,
    ) -> Callable[
        [accessapproval.DismissApprovalRequestMessage], accessapproval.ApprovalRequest
    ]:
        r"""Return a callable for the dismiss approval request method over gRPC.

        Dismisses a request. Returns the updated ApprovalRequest.

        NOTE: This does not deny access to the resource if another
        request has been made and approved. It is equivalent in effect
        to ignoring the request altogether.

        Returns NOT_FOUND if the request does not exist.

        Returns FAILED_PRECONDITION if the request exists but is not in
        a pending state.

        Returns:
            Callable[[~.DismissApprovalRequestMessage],
                    ~.ApprovalRequest]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "dismiss_approval_request" not in self._stubs:
            self._stubs["dismiss_approval_request"] = self.grpc_channel.unary_unary(
                "/google.cloud.accessapproval.v1.AccessApproval/DismissApprovalRequest",
                request_serializer=accessapproval.DismissApprovalRequestMessage.serialize,
                response_deserializer=accessapproval.ApprovalRequest.deserialize,
            )
        return self._stubs["dismiss_approval_request"]

    @property
    def get_access_approval_settings(
        self,
    ) -> Callable[
        [accessapproval.GetAccessApprovalSettingsMessage],
        accessapproval.AccessApprovalSettings,
    ]:
        r"""Return a callable for the get access approval settings method over gRPC.

        Gets the settings associated with a project, folder,
        or organization.

        Returns:
            Callable[[~.GetAccessApprovalSettingsMessage],
                    ~.AccessApprovalSettings]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "get_access_approval_settings" not in self._stubs:
            self._stubs["get_access_approval_settings"] = self.grpc_channel.unary_unary(
                "/google.cloud.accessapproval.v1.AccessApproval/GetAccessApprovalSettings",
                request_serializer=accessapproval.GetAccessApprovalSettingsMessage.serialize,
                response_deserializer=accessapproval.AccessApprovalSettings.deserialize,
            )
        return self._stubs["get_access_approval_settings"]

    @property
    def update_access_approval_settings(
        self,
    ) -> Callable[
        [accessapproval.UpdateAccessApprovalSettingsMessage],
        accessapproval.AccessApprovalSettings,
    ]:
        r"""Return a callable for the update access approval
        settings method over gRPC.

        Updates the settings associated with a project, folder, or
        organization. Settings to update are determined by the value of
        field_mask.

        Returns:
            Callable[[~.UpdateAccessApprovalSettingsMessage],
                    ~.AccessApprovalSettings]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "update_access_approval_settings" not in self._stubs:
            self._stubs[
                "update_access_approval_settings"
            ] = self.grpc_channel.unary_unary(
                "/google.cloud.accessapproval.v1.AccessApproval/UpdateAccessApprovalSettings",
                request_serializer=accessapproval.UpdateAccessApprovalSettingsMessage.serialize,
                response_deserializer=accessapproval.AccessApprovalSettings.deserialize,
            )
        return self._stubs["update_access_approval_settings"]

    @property
    def delete_access_approval_settings(
        self,
    ) -> Callable[
        [accessapproval.DeleteAccessApprovalSettingsMessage], empty_pb2.Empty
    ]:
        r"""Return a callable for the delete access approval
        settings method over gRPC.

        Deletes the settings associated with a project,
        folder, or organization. This will have the effect of
        disabling Access Approval for the project, folder, or
        organization, but only if all ancestors also have Access
        Approval disabled. If Access Approval is enabled at a
        higher level of the hierarchy, then Access Approval will
        still be enabled at this level as the settings are
        inherited.

        Returns:
            Callable[[~.DeleteAccessApprovalSettingsMessage],
                    ~.Empty]:
                A function that, when called, will call the underlying RPC
                on the server.
        """
        # Generate a "stub function" on-the-fly which will actually make
        # the request.
        # gRPC handles serialization and deserialization, so we just need
        # to pass in the functions for each.
        if "delete_access_approval_settings" not in self._stubs:
            self._stubs[
                "delete_access_approval_settings"
            ] = self.grpc_channel.unary_unary(
                "/google.cloud.accessapproval.v1.AccessApproval/DeleteAccessApprovalSettings",
                request_serializer=accessapproval.DeleteAccessApprovalSettingsMessage.serialize,
                response_deserializer=empty_pb2.Empty.FromString,
            )
        return self._stubs["delete_access_approval_settings"]

    def close(self):
        self.grpc_channel.close()


__all__ = ("AccessApprovalGrpcTransport",)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import os
import re
from typing import Dict, Optional, Sequence, Tuple, Type, Union
import pkg_resources

from google.api_core import client_options as client_options_lib
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport import mtls  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.auth.exceptions import MutualTLSChannelError  # type: ignore
from google.oauth2 import service_account  # type: ignore

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object]  # type: ignore

from google.cloud.accessapproval_v1.services.access_approval import pagers
from google.cloud.accessapproval_v1.types import accessapproval
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore
from .transports.base import AccessApprovalTransport, DEFAULT_CLIENT_INFO
from .transports.grpc import AccessApprovalGrpcTransport
from .transports.grpc_asyncio import AccessApprovalGrpcAsyncIOTransport


class AccessApprovalClientMeta(type):
    """Metaclass for the AccessApproval client.

    This provides class-level methods for building and retrieving
    support objects (e.g. transport) without polluting the client instance
    objects.
    """

    _transport_registry = (
        OrderedDict()
    )  # type: Dict[str, Type[AccessApprovalTransport]]
    _transport_registry["grpc"] = AccessApprovalGrpcTransport
    _transport_registry["grpc_asyncio"] = AccessApprovalGrpcAsyncIOTransport

    def get_transport_class(cls, label: str = None,) -> Type[AccessApprovalTransport]:
        """Returns an appropriate transport class.

        Args:
            label: The name of the desired transport. If none is
                provided, then the first transport in the registry is used.

        Returns:
            The transport class to use.
        """
        # If a specific transport is requested, return that one.
        if label:
            return cls._transport_registry[label]

        # No transport is requested; return the default (that is, the first one
        # in the dictionary).
        return next(iter(cls._transport_registry.values()))


class AccessApprovalClient(metaclass=AccessApprovalClientMeta):
    """This API allows a customer to manage accesses to cloud resources by
    Google personnel. It defines the following resource model:

    -  The API has a collection of
       [ApprovalRequest][google.cloud.accessapproval.v1.ApprovalRequest]
       resources, named ``approvalRequests/{approval_request}``
    -  The API has top-level settings per Project/Folder/Organization,
       named ``accessApprovalSettings``

    The service also periodically emails a list of recipients, defined
    at the Project/Folder/Organization level in the
    accessApprovalSettings, when there is a pending ApprovalRequest for
    them to act on. The ApprovalRequests can also optionally be
    published to a Pub/Sub topic owned by the customer (contact support
    if you would like to enable Pub/Sub notifications).

    ApprovalRequests can be approved or dismissed. Google personnel can
    only access the indicated resource or resources if the request is
    approved (subject to some exclusions:
    https://cloud.google.com/access-approval/docs/overview#exclusions).

    Note: Using Access Approval functionality will mean that Google may
    not be able to meet the SLAs for your chosen products, as any
    support response times may be dramatically increased. As such the
    SLAs do not apply to any service disruption to the extent impacted
    by Customer's use of Access Approval. Do not enable Access Approval
    for projects where you may require high service availability and
    rapid response by Google Cloud Support.

    After a request is approved or dismissed, no further action may be
    taken on it. Requests with the requested_expiration in the past or
    with no activity for 14 days are considered dismissed. When an
    approval expires, the request is considered dismissed.

    If a request is not approved or dismissed, we call it pending.
    """

    @staticmethod
    def _get_default_mtls_endpoint(api_endpoint):
        """Converts api endpoint to mTLS endpoint.

        Convert "*.sandbox.googleapis.com" and "*.googleapis.com" to
        "*.mtls.sandbox.googleapis.com" and "*.mtls.googleapis.com" respectively.
        Args:
            api_endpoint (Optional[str]): the api endpoint to convert.
        Returns:
            str: converted mTLS api endpoint.
        """
        if not api_endpoint:
            return api_endpoint

        mtls_endpoint_re = re.compile(
            r"(?P<name>[^.]+)(?P<mtls>\.mtls)?(?P<sandbox>\.sandbox)?(?P<googledomain>\.googleapis\.com)?"
        )

        m = mtls_endpoint_re.match(api_endpoint)
        name, mtls, sandbox, googledomain = m.groups()
        if mtls or not googledomain:
            return api_endpoint

        if sandbox:
            return api_endpoint.replace(
                "sandbox.googleapis.com", "mtls.sandbox.googleapis.com"
            )

        return api_endpoint.replace(".googleapis.com", ".mtls.googleapis.com")

    DEFAULT_ENDPOINT = "accessapproval.googleapis.com"
    DEFAULT_MTLS_ENDPOINT = _get_default_mtls_endpoint.__func__(  # type: ignore
        DEFAULT_ENDPOINT
    )

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            AccessApprovalClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_info(info)
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            AccessApprovalClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_file(filename)
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> AccessApprovalTransport:
        """Returns the transport used by the client instance.

        Returns:
            AccessApprovalTransport: The transport used by the client
                instance.
        """
        return self._transport

    @staticmethod
    def access_approval_settings_path(project: str,) -> str:
        """Returns a fully-qualified access_approval_settings string."""
        return "projects/{project}/accessApprovalSettings".format(project=project,)

    @staticmethod
    def parse_access_approval_settings_path(path: str) -> Dict[str, str]:
        """Parses a access_approval_settings path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)/accessApprovalSettings$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def approval_request_path(project: str, approval_request: str,) -> str:
        """Returns a fully-qualified approval_request string."""
        return "projects/{project}/approvalRequests/{approval_request}".format(
            project=project, approval_request=approval_request,
        )

    @staticmethod
    def parse_approval_request_path(path: str) -> Dict[str, str]:
        """Parses a approval_request path into its component segments."""
        m = re.match(
            r"^projects/(?P<project>.+?)/approvalRequests/(?P<approval_request>.+?)$",
            path,
        )
        return m.groupdict() if m else {}

    @staticmethod
    def common_billing_account_path(billing_account: str,) -> str:
        """Returns a fully-qualified billing_account string."""
        return "billingAccounts/{billing_account}".format(
            billing_account=billing_account,
        )

    @staticmethod
    def parse_common_billing_account_path(path: str) -> Dict[str, str]:
        """Parse a billing_account path into its component segments."""
        m = re.match(r"^billingAccounts/(?P<billing_account>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_folder_path(folder: str,) -> str:
        """Returns a fully-qualified folder string."""
        return "folders/{folder}".format(folder=folder,)

    @staticmethod
    def parse_common_folder_path(path: str) -> Dict[str, str]:
        """Parse a folder path into its component segments."""
        m = re.match(r"^folders/(?P<folder>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_organization_path(organization: str,) -> str:
        """Returns a fully-qualified organization string."""
        return "organizations/{organization}".format(organization=organization,)

    @staticmethod
    def parse_common_organization_path(path: str) -> Dict[str, str]:
        """Parse a organization path into its component segments."""
        m = re.match(r"^organizations/(?P<organization>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_project_path(project: str,) -> str:
        """Returns a fully-qualified project string."""
        return "projects/{project}".format(project=project,)

    @staticmethod
    def parse_common_project_path(path: str) -> Dict[str, str]:
        """Parse a project path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_location_path(project: str, location: str,) -> str:
        """Returns a fully-qualified location string."""
        return "projects/{project}/locations/{location}".format(
            project=project, location=location,
        )

    @staticmethod
    def parse_common_location_path(path: str) -> Dict[str, str]:
        """Parse a location path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
        return m.groupdict() if m else {}

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[client_options_lib.ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variabel is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        if client_options is None:
            client_options = client_options_lib.ClientOptions()
        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
        if use_client_cert not in ("true", "false"):
            raise ValueError(
                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
            )
        if use_mtls_endpoint not in ("auto", "never", "always"):
            raise MutualTLSChannelError(
                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
            )

        # Figure out the client cert source to use.
        client_cert_source = None
        if use_client_cert == "true":
            if client_options.client_cert_source:
                client_cert_source = client_options.client_cert_source
            elif mtls.has_default_client_cert_source():
                client_cert_source = mtls.default_client_cert_source()

        # Figure out which api endpoint to use.
        if client_options.api_endpoint is not None:
            api_endpoint = client_options.api_endpoint
        elif use_mtls_endpoint == "always" or (
            use_mtls_endpoint == "auto" and client_cert_source
        ):
            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
        else:
            api_endpoint = cls.DEFAULT_ENDPOINT

        return api_endpoint, client_cert_source

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Union[str, AccessApprovalTransport, None] = None,
        client_options: Optional[client_options_lib.ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the access approval client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Union[str, AccessApprovalTransport]): The
                transport to use. If set to None, a transport is chosen
                automatically.
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. It won't take effect if a ``transport`` instance is provided.
                (1) The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
                environment variable can also be used to override the endpoint:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto switch to the
                default mTLS endpoint if client certificate is present, this is
                the default value). However, the ``api_endpoint`` property takes
                precedence if provided.
                (2) If GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide client certificate for mutual TLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        if isinstance(client_options, dict):
            client_options = client_options_lib.from_dict(client_options)
        if client_options is None:
            client_options = client_options_lib.ClientOptions()

        api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
            client_options
        )

        api_key_value = getattr(client_options, "api_key", None)
        if api_key_value and credentials:
            raise ValueError(
                "client_options.api_key and credentials are mutually exclusive"
            )

        # Save or instantiate the transport.
        # Ordinarily, we provide the transport, but allowing a custom transport
        # instance provides an extensibility point for unusual situations.
        if isinstance(transport, AccessApprovalTransport):
            # transport is a AccessApprovalTransport instance.
            if credentials or client_options.credentials_file or api_key_value:
                raise ValueError(
                    "When providing a transport instance, "
                    "provide its credentials directly."
                )
            if client_options.scopes:
                raise ValueError(
                    "When providing a transport instance, provide its scopes "
                    "directly."
                )
            self._transport = transport
        else:
            import google.auth._default  # type: ignore

            if api_key_value and hasattr(
                google.auth._default, "get_api_key_credentials"
            ):
                credentials = google.auth._default.get_api_key_credentials(
                    api_key_value
                )

            Transport = type(self).get_transport_class(transport)
            self._transport = Transport(
                credentials=credentials,
                credentials_file=client_options.credentials_file,
                host=api_endpoint,
                scopes=client_options.scopes,
                client_cert_source_for_mtls=client_cert_source_func,
                quota_project_id=client_options.quota_project_id,
                client_info=client_info,
                always_use_jwt_access=True,
            )

    def list_approval_requests(
        self,
        request: Union[accessapproval.ListApprovalRequestsMessage, dict] = None,
        *,
        parent: str = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> pagers.ListApprovalRequestsPager:
        r"""Lists approval requests associated with a project,
        folder, or organization. Approval requests can be
        filtered by state (pending, active, dismissed). The
        order is reverse chronological.



        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_list_approval_requests():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.ListApprovalRequestsMessage(
                )

                # Make the request
                page_result = client.list_approval_requests(request=request)

                # Handle the response
                for response in page_result:
                    print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.ListApprovalRequestsMessage, dict]):
                The request object. Request to list approval requests.
            parent (str):
                The parent resource. This may be
                "projects/{project}",
                "folders/{folder}", or
                "organizations/{organization}".

                This corresponds to the ``parent`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.services.access_approval.pagers.ListApprovalRequestsPager:
                Response to listing of
                ApprovalRequest objects.
                Iterating over this object will yield
                results and resolve additional pages
                automatically.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([parent])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a accessapproval.ListApprovalRequestsMessage.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, accessapproval.ListApprovalRequestsMessage):
            request = accessapproval.ListApprovalRequestsMessage(request)
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if parent is not None:
                request.parent = parent

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.list_approval_requests]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("parent", request.parent),)),
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # This method is paged; wrap the response in a pager, which provides
        # an `__iter__` convenience method.
        response = pagers.ListApprovalRequestsPager(
            method=rpc, request=request, response=response, metadata=metadata,
        )

        # Done; return the response.
        return response

    def get_approval_request(
        self,
        request: Union[accessapproval.GetApprovalRequestMessage, dict] = None,
        *,
        name: str = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> accessapproval.ApprovalRequest:
        r"""Gets an approval request. Returns NOT_FOUND if the request does
        not exist.



        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_get_approval_request():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.GetApprovalRequestMessage(
                )

                # Make the request
                response = client.get_approval_request(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.GetApprovalRequestMessage, dict]):
                The request object. Request to get an approval request.
            name (str):
                The name of the approval request to retrieve. Format:
                "{projects|folders|organizations}/{id}/approvalRequests/{approval_request}"

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.types.ApprovalRequest:
                A request for the customer to approve
                access to a resource.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a accessapproval.GetApprovalRequestMessage.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, accessapproval.GetApprovalRequestMessage):
            request = accessapproval.GetApprovalRequestMessage(request)
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if name is not None:
                request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.get_approval_request]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    def approve_approval_request(
        self,
        request: Union[accessapproval.ApproveApprovalRequestMessage, dict] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> accessapproval.ApprovalRequest:
        r"""Approves a request and returns the updated ApprovalRequest.

        Returns NOT_FOUND if the request does not exist. Returns
        FAILED_PRECONDITION if the request exists but is not in a
        pending state.



        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_approve_approval_request():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.ApproveApprovalRequestMessage(
                )

                # Make the request
                response = client.approve_approval_request(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.ApproveApprovalRequestMessage, dict]):
                The request object. Request to approve an
                ApprovalRequest.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.types.ApprovalRequest:
                A request for the customer to approve
                access to a resource.

        """
        # Create or coerce a protobuf request object.
        # Minor optimization to avoid making a copy if the user passes
        # in a accessapproval.ApproveApprovalRequestMessage.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, accessapproval.ApproveApprovalRequestMessage):
            request = accessapproval.ApproveApprovalRequestMessage(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.approve_approval_request]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    def dismiss_approval_request(
        self,
        request: Union[accessapproval.DismissApprovalRequestMessage, dict] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> accessapproval.ApprovalRequest:
        r"""Dismisses a request. Returns the updated ApprovalRequest.

        NOTE: This does not deny access to the resource if another
        request has been made and approved. It is equivalent in effect
        to ignoring the request altogether.

        Returns NOT_FOUND if the request does not exist.

        Returns FAILED_PRECONDITION if the request exists but is not in
        a pending state.



        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_dismiss_approval_request():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.DismissApprovalRequestMessage(
                )

                # Make the request
                response = client.dismiss_approval_request(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.DismissApprovalRequestMessage, dict]):
                The request object. Request to dismiss an approval
                request.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.types.ApprovalRequest:
                A request for the customer to approve
                access to a resource.

        """
        # Create or coerce a protobuf request object.
        # Minor optimization to avoid making a copy if the user passes
        # in a accessapproval.DismissApprovalRequestMessage.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, accessapproval.DismissApprovalRequestMessage):
            request = accessapproval.DismissApprovalRequestMessage(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.dismiss_approval_request]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    def get_access_approval_settings(
        self,
        request: Union[accessapproval.GetAccessApprovalSettingsMessage, dict] = None,
        *,
        name: str = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> accessapproval.AccessApprovalSettings:
        r"""Gets the settings associated with a project, folder,
        or organization.



        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_get_access_approval_settings():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.GetAccessApprovalSettingsMessage(
                )

                # Make the request
                response = client.get_access_approval_settings(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.GetAccessApprovalSettingsMessage, dict]):
                The request object. Request to get access approval
                settings.
            name (str):
                The name of the AccessApprovalSettings to retrieve.
                Format:
                "{projects|folders|organizations}/{id}/accessApprovalSettings"

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.types.AccessApprovalSettings:
                Settings on a
                Project/Folder/Organization related to
                Access Approval.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a accessapproval.GetAccessApprovalSettingsMessage.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, accessapproval.GetAccessApprovalSettingsMessage):
            request = accessapproval.GetAccessApprovalSettingsMessage(request)
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if name is not None:
                request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[
            self._transport.get_access_approval_settings
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    def update_access_approval_settings(
        self,
        request: Union[accessapproval.UpdateAccessApprovalSettingsMessage, dict] = None,
        *,
        settings: accessapproval.AccessApprovalSettings = None,
        update_mask: field_mask_pb2.FieldMask = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> accessapproval.AccessApprovalSettings:
        r"""Updates the settings associated with a project, folder, or
        organization. Settings to update are determined by the value of
        field_mask.



        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_update_access_approval_settings():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.UpdateAccessApprovalSettingsMessage(
                )

                # Make the request
                response = client.update_access_approval_settings(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.UpdateAccessApprovalSettingsMessage, dict]):
                The request object. Request to update access approval
                settings.
            settings (google.cloud.accessapproval_v1.types.AccessApprovalSettings):
                The new AccessApprovalSettings.
                This corresponds to the ``settings`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            update_mask (google.protobuf.field_mask_pb2.FieldMask):
                The update mask applies to the settings. Only the top
                level fields of AccessApprovalSettings
                (notification_emails & enrolled_services) are supported.
                For each field, if it is included, the currently stored
                value will be entirely overwritten with the value of the
                field passed in this request.

                For the ``FieldMask`` definition, see
                https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#fieldmask
                If this field is left unset, only the
                notification_emails field will be updated.

                This corresponds to the ``update_mask`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.

        Returns:
            google.cloud.accessapproval_v1.types.AccessApprovalSettings:
                Settings on a
                Project/Folder/Organization related to
                Access Approval.

        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([settings, update_mask])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a accessapproval.UpdateAccessApprovalSettingsMessage.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, accessapproval.UpdateAccessApprovalSettingsMessage):
            request = accessapproval.UpdateAccessApprovalSettingsMessage(request)
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if settings is not None:
                request.settings = settings
            if update_mask is not None:
                request.update_mask = update_mask

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[
            self._transport.update_access_approval_settings
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("settings.name", request.settings.name),)
            ),
        )

        # Send the request.
        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)

        # Done; return the response.
        return response

    def delete_access_approval_settings(
        self,
        request: Union[accessapproval.DeleteAccessApprovalSettingsMessage, dict] = None,
        *,
        name: str = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: float = None,
        metadata: Sequence[Tuple[str, str]] = (),
    ) -> None:
        r"""Deletes the settings associated with a project,
        folder, or organization. This will have the effect of
        disabling Access Approval for the project, folder, or
        organization, but only if all ancestors also have Access
        Approval disabled. If Access Approval is enabled at a
        higher level of the hierarchy, then Access Approval will
        still be enabled at this level as the settings are
        inherited.



        .. code-block::

            from google.cloud import accessapproval_v1

            def sample_delete_access_approval_settings():
                # Create a client
                client = accessapproval_v1.AccessApprovalClient()

                # Initialize request argument(s)
                request = accessapproval_v1.DeleteAccessApprovalSettingsMessage(
                )

                # Make the request
                client.delete_access_approval_settings(request=request)

        Args:
            request (Union[google.cloud.accessapproval_v1.types.DeleteAccessApprovalSettingsMessage, dict]):
                The request object. Request to delete access approval
                settings.
            name (str):
                Name of the AccessApprovalSettings to
                delete.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, str]]): Strings which should be
                sent along with the request as metadata.
        """
        # Create or coerce a protobuf request object.
        # Quick check: If we got a request object, we should *not* have
        # gotten any keyword arguments that map to the request.
        has_flattened_params = any([name])
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # Minor optimization to avoid making a copy if the user passes
        # in a accessapproval.DeleteAccessApprovalSettingsMessage.
        # There's no risk of modifying the input as we've already verified
        # there are no flattened fields.
        if not isinstance(request, accessapproval.DeleteAccessApprovalSettingsMessage):
            request = accessapproval.DeleteAccessApprovalSettingsMessage(request)
            # If we have keyword arguments corresponding to fields on the
            # request, apply these.
            if name is not None:
                request.name = name

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[
            self._transport.delete_access_approval_settings
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata((("name", request.name),)),
        )

        # Send the request.
        rpc(
            request, retry=retry, timeout=timeout, metadata=metadata,
        )

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        """Releases underlying transport's resources.

        .. warning::
            ONLY use as a context manager if the transport is NOT shared
            with other clients! Exiting the with block will CLOSE the transport
            and may cause errors in other clients!
        """
        self.transport.close()


try:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
        gapic_version=pkg_resources.get_distribution(
            "google-cloud-access-approval",
        ).version,
    )
except pkg_resources.DistributionNotFound:
    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()


__all__ = ("AccessApprovalClient",)



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import proto  # type: ignore

from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.accessapproval.v1",
    manifest={
        "EnrollmentLevel",
        "AccessLocations",
        "AccessReason",
        "ApproveDecision",
        "DismissDecision",
        "ResourceProperties",
        "ApprovalRequest",
        "EnrolledService",
        "AccessApprovalSettings",
        "ListApprovalRequestsMessage",
        "ListApprovalRequestsResponse",
        "GetApprovalRequestMessage",
        "ApproveApprovalRequestMessage",
        "DismissApprovalRequestMessage",
        "GetAccessApprovalSettingsMessage",
        "UpdateAccessApprovalSettingsMessage",
        "DeleteAccessApprovalSettingsMessage",
    },
)


class EnrollmentLevel(proto.Enum):
    r"""Represents the type of enrollment for a given service to
    Access Approval.
    """
    ENROLLMENT_LEVEL_UNSPECIFIED = 0
    BLOCK_ALL = 1


class AccessLocations(proto.Message):
    r"""Home office and physical location of the principal.

    Attributes:
        principal_office_country (str):
            The "home office" location of the principal. A two-letter
            country code (ISO 3166-1 alpha-2), such as "US", "DE" or
            "GB" or a region code. In some limited situations Google
            systems may refer refer to a region code instead of a
            country code. Possible Region Codes:

            -  ASI: Asia
            -  EUR: Europe
            -  OCE: Oceania
            -  AFR: Africa
            -  NAM: North America
            -  SAM: South America
            -  ANT: Antarctica
            -  ANY: Any location
        principal_physical_location_country (str):
            Physical location of the principal at the time of the
            access. A two-letter country code (ISO 3166-1 alpha-2), such
            as "US", "DE" or "GB" or a region code. In some limited
            situations Google systems may refer refer to a region code
            instead of a country code. Possible Region Codes:

            -  ASI: Asia
            -  EUR: Europe
            -  OCE: Oceania
            -  AFR: Africa
            -  NAM: North America
            -  SAM: South America
            -  ANT: Antarctica
            -  ANY: Any location
    """

    principal_office_country = proto.Field(proto.STRING, number=1,)
    principal_physical_location_country = proto.Field(proto.STRING, number=2,)


class AccessReason(proto.Message):
    r"""

    Attributes:
        type_ (google.cloud.accessapproval_v1.types.AccessReason.Type):
            Type of access justification.
        detail (str):
            More detail about certain reason types. See
            comments for each type above.
    """

    class Type(proto.Enum):
        r"""Type of access justification."""
        TYPE_UNSPECIFIED = 0
        CUSTOMER_INITIATED_SUPPORT = 1
        GOOGLE_INITIATED_SERVICE = 2
        GOOGLE_INITIATED_REVIEW = 3

    type_ = proto.Field(proto.ENUM, number=1, enum=Type,)
    detail = proto.Field(proto.STRING, number=2,)


class ApproveDecision(proto.Message):
    r"""A decision that has been made to approve access to a
    resource.

    Attributes:
        approve_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which approval was granted.
        expire_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the approval expires.
    """

    approve_time = proto.Field(
        proto.MESSAGE, number=1, message=timestamp_pb2.Timestamp,
    )
    expire_time = proto.Field(proto.MESSAGE, number=2, message=timestamp_pb2.Timestamp,)


class DismissDecision(proto.Message):
    r"""A decision that has been made to dismiss an approval request.

    Attributes:
        dismiss_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the approval request was
            dismissed.
        implicit (bool):
            This field will be true if the
            ApprovalRequest was implcitly dismissed due to
            inaction by the access approval approvers (the
            request is not acted on by the approvers before
            the exiration time).
    """

    dismiss_time = proto.Field(
        proto.MESSAGE, number=1, message=timestamp_pb2.Timestamp,
    )
    implicit = proto.Field(proto.BOOL, number=2,)


class ResourceProperties(proto.Message):
    r"""The properties associated with the resource of the request.

    Attributes:
        excludes_descendants (bool):
            Whether an approval will exclude the
            descendants of the resource being requested.
    """

    excludes_descendants = proto.Field(proto.BOOL, number=1,)


class ApprovalRequest(proto.Message):
    r"""A request for the customer to approve access to a resource.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        name (str):
            The resource name of the request. Format is
            "{projects|folders|organizations}/{id}/approvalRequests/{approval_request}".
        requested_resource_name (str):
            The resource for which approval is being requested. The
            format of the resource name is defined at
            https://cloud.google.com/apis/design/resource_names. The
            resource name here may either be a "full" resource name
            (e.g. "//library.googleapis.com/shelves/shelf1/books/book2")
            or a "relative" resource name (e.g.
            "shelves/shelf1/books/book2") as described in the resource
            name specification.
        requested_resource_properties (google.cloud.accessapproval_v1.types.ResourceProperties):
            Properties related to the resource represented by
            requested_resource_name.
        requested_reason (google.cloud.accessapproval_v1.types.AccessReason):
            The justification for which approval is being
            requested.
        requested_locations (google.cloud.accessapproval_v1.types.AccessLocations):
            The locations for which approval is being
            requested.
        request_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which approval was requested.
        requested_expiration (google.protobuf.timestamp_pb2.Timestamp):
            The requested expiration for the approval. If
            the request is approved, access will be granted
            from the time of approval until the expiration
            time.
        approve (google.cloud.accessapproval_v1.types.ApproveDecision):
            Access was approved.

            This field is a member of `oneof`_ ``decision``.
        dismiss (google.cloud.accessapproval_v1.types.DismissDecision):
            The request was dismissed.

            This field is a member of `oneof`_ ``decision``.
    """

    name = proto.Field(proto.STRING, number=1,)
    requested_resource_name = proto.Field(proto.STRING, number=2,)
    requested_resource_properties = proto.Field(
        proto.MESSAGE, number=9, message="ResourceProperties",
    )
    requested_reason = proto.Field(proto.MESSAGE, number=3, message="AccessReason",)
    requested_locations = proto.Field(
        proto.MESSAGE, number=4, message="AccessLocations",
    )
    request_time = proto.Field(
        proto.MESSAGE, number=5, message=timestamp_pb2.Timestamp,
    )
    requested_expiration = proto.Field(
        proto.MESSAGE, number=6, message=timestamp_pb2.Timestamp,
    )
    approve = proto.Field(
        proto.MESSAGE, number=7, oneof="decision", message="ApproveDecision",
    )
    dismiss = proto.Field(
        proto.MESSAGE, number=8, oneof="decision", message="DismissDecision",
    )


class EnrolledService(proto.Message):
    r"""Represents the enrollment of a cloud resource into a specific
    service.

    Attributes:
        cloud_product (str):
            The product for which Access Approval will be enrolled.
            Allowed values are listed below (case-sensitive):

            -  all
            -  GA
            -  App Engine
            -  BigQuery
            -  Cloud Bigtable
            -  Cloud Key Management Service
            -  Compute Engine
            -  Cloud Dataflow
            -  Cloud DLP
            -  Cloud EKM
            -  Cloud HSM
            -  Cloud Identity and Access Management
            -  Cloud Logging
            -  Cloud Pub/Sub
            -  Cloud Spanner
            -  Cloud SQL
            -  Cloud Storage
            -  Google Kubernetes Engine
            -  Organization Policy Serivice
            -  Persistent Disk
            -  Resource Manager
            -  Speaker ID

            Note: These values are supported as input for legacy
            purposes, but will not be returned from the API.

            -  all
            -  ga-only
            -  appengine.googleapis.com
            -  bigquery.googleapis.com
            -  bigtable.googleapis.com
            -  container.googleapis.com
            -  cloudkms.googleapis.com
            -  cloudresourcemanager.googleapis.com
            -  cloudsql.googleapis.com
            -  compute.googleapis.com
            -  dataflow.googleapis.com
            -  dlp.googleapis.com
            -  iam.googleapis.com
            -  logging.googleapis.com
            -  orgpolicy.googleapis.com
            -  pubsub.googleapis.com
            -  spanner.googleapis.com
            -  speakerid.googleapis.com
            -  storage.googleapis.com

            Calls to UpdateAccessApprovalSettings using 'all' or any of
            the XXX.googleapis.com will be translated to the associated
            product name ('all', 'App Engine', etc.).

            Note: 'all' will enroll the resource in all products
            supported at both 'GA' and 'Preview' levels.

            More information about levels of support is available at
            https://cloud.google.com/access-approval/docs/supported-services
        enrollment_level (google.cloud.accessapproval_v1.types.EnrollmentLevel):
            The enrollment level of the service.
    """

    cloud_product = proto.Field(proto.STRING, number=1,)
    enrollment_level = proto.Field(proto.ENUM, number=2, enum="EnrollmentLevel",)


class AccessApprovalSettings(proto.Message):
    r"""Settings on a Project/Folder/Organization related to Access
    Approval.

    Attributes:
        name (str):
            The resource name of the settings. Format is one of:

            -  "projects/{project}/accessApprovalSettings"
            -  "folders/{folder}/accessApprovalSettings"
            -  "organizations/{organization}/accessApprovalSettings".
        notification_emails (Sequence[str]):
            A list of email addresses to which
            notifications relating to approval requests
            should be sent. Notifications relating to a
            resource will be sent to all emails in the
            settings of ancestor resources of that resource.
            A maximum of 50 email addresses are allowed.
        enrolled_services (Sequence[google.cloud.accessapproval_v1.types.EnrolledService]):
            A list of Google Cloud Services for which the given resource
            has Access Approval enrolled. Access requests for the
            resource given by name against any of these services
            contained here will be required to have explicit approval.
            If name refers to an organization, enrollment can be done
            for individual services. If name refers to a folder or
            project, enrollment can only be done on an all or nothing
            basis.

            If a cloud_product is repeated in this list, the first entry
            will be honored and all following entries will be discarded.
            A maximum of 10 enrolled services will be enforced, to be
            expanded as the set of supported services is expanded.
        enrolled_ancestor (bool):
            Output only. This field is read only (not
            settable via UpdateAccessApprovalSettings
            method). If the field is true, that indicates
            that at least one service is enrolled for Access
            Approval in one or more ancestors of the Project
            or Folder (this field will always be unset for
            the organization since organizations do not have
            ancestors).
    """

    name = proto.Field(proto.STRING, number=1,)
    notification_emails = proto.RepeatedField(proto.STRING, number=2,)
    enrolled_services = proto.RepeatedField(
        proto.MESSAGE, number=3, message="EnrolledService",
    )
    enrolled_ancestor = proto.Field(proto.BOOL, number=4,)


class ListApprovalRequestsMessage(proto.Message):
    r"""Request to list approval requests.

    Attributes:
        parent (str):
            The parent resource. This may be
            "projects/{project}", "folders/{folder}", or
            "organizations/{organization}".
        filter (str):
            A filter on the type of approval requests to retrieve. Must
            be one of the following values:

            -  [not set]: Requests that are pending or have active
               approvals.
            -  ALL: All requests.
            -  PENDING: Only pending requests.
            -  ACTIVE: Only active (i.e. currently approved) requests.
            -  DISMISSED: Only requests that have been dismissed, or
               requests that are not approved and past expiration.
            -  EXPIRED: Only requests that have been approved, and the
               approval has expired.
            -  HISTORY: Active, dismissed and expired requests.
        page_size (int):
            Requested page size.
        page_token (str):
            A token identifying the page of results to
            return.
    """

    parent = proto.Field(proto.STRING, number=1,)
    filter = proto.Field(proto.STRING, number=2,)
    page_size = proto.Field(proto.INT32, number=3,)
    page_token = proto.Field(proto.STRING, number=4,)


class ListApprovalRequestsResponse(proto.Message):
    r"""Response to listing of ApprovalRequest objects.

    Attributes:
        approval_requests (Sequence[google.cloud.accessapproval_v1.types.ApprovalRequest]):
            Approval request details.
        next_page_token (str):
            Token to retrieve the next page of results,
            or empty if there are no more.
    """

    @property
    def raw_page(self):
        return self

    approval_requests = proto.RepeatedField(
        proto.MESSAGE, number=1, message="ApprovalRequest",
    )
    next_page_token = proto.Field(proto.STRING, number=2,)


class GetApprovalRequestMessage(proto.Message):
    r"""Request to get an approval request.

    Attributes:
        name (str):
            The name of the approval request to retrieve. Format:
            "{projects|folders|organizations}/{id}/approvalRequests/{approval_request}".
    """

    name = proto.Field(proto.STRING, number=1,)


class ApproveApprovalRequestMessage(proto.Message):
    r"""Request to approve an ApprovalRequest.

    Attributes:
        name (str):
            Name of the approval request to approve.
        expire_time (google.protobuf.timestamp_pb2.Timestamp):
            The expiration time of this approval.
    """

    name = proto.Field(proto.STRING, number=1,)
    expire_time = proto.Field(proto.MESSAGE, number=2, message=timestamp_pb2.Timestamp,)


class DismissApprovalRequestMessage(proto.Message):
    r"""Request to dismiss an approval request.

    Attributes:
        name (str):
            Name of the ApprovalRequest to dismiss.
    """

    name = proto.Field(proto.STRING, number=1,)


class GetAccessApprovalSettingsMessage(proto.Message):
    r"""Request to get access approval settings.

    Attributes:
        name (str):
            The name of the AccessApprovalSettings to retrieve. Format:
            "{projects|folders|organizations}/{id}/accessApprovalSettings".
    """

    name = proto.Field(proto.STRING, number=1,)


class UpdateAccessApprovalSettingsMessage(proto.Message):
    r"""Request to update access approval settings.

    Attributes:
        settings (google.cloud.accessapproval_v1.types.AccessApprovalSettings):
            The new AccessApprovalSettings.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            The update mask applies to the settings. Only the top level
            fields of AccessApprovalSettings (notification_emails &
            enrolled_services) are supported. For each field, if it is
            included, the currently stored value will be entirely
            overwritten with the value of the field passed in this
            request.

            For the ``FieldMask`` definition, see
            https://developers.google.com/protocol-buffers/docs/reference/google.protobuf#fieldmask
            If this field is left unset, only the notification_emails
            field will be updated.
    """

    settings = proto.Field(proto.MESSAGE, number=1, message="AccessApprovalSettings",)
    update_mask = proto.Field(
        proto.MESSAGE, number=2, message=field_mask_pb2.FieldMask,
    )


class DeleteAccessApprovalSettingsMessage(proto.Message):
    r"""Request to delete access approval settings.

    Attributes:
        name (str):
            Name of the AccessApprovalSettings to delete.
    """

    name = proto.Field(proto.STRING, number=1,)


__all__ = tuple(sorted(__protobuf__.manifest))
